// ATtiny1614 pins definitions
#define NEOPIXEL_PIN 1  // Pin connected to the NeoPixel strip (PA3)
#define NUM_LEDS 3      // Number of NeoPixel LEDs in the strip

// Function prototypes
void neopixelSetColor(uint8_t r, uint8_t g, uint8_t b);
void neopixelShow();
void neopixelClear();

void setup() {
  // Set NeoPixel pin as output
  PORTA.DIRSET = PIN5_bm;

  // Initialize NeoPixel
  neopixelClear();
  neopixelShow();
}

void loop() {
  // Fill the first LED with red color
  neopixelSetColor(255, 0, 0);
  neopixelShow();
  delay(500);

  // Fill the second LED with green color
  neopixelSetColor(0, 255, 0);
  neopixelShow();
  delay(500);

  // Fill both LEDs with blue color
  neopixelSetColor(0, 0, 255);
  neopixelShow();
  delay(500);
}

// Function to set the color of the NeoPixel LEDs
void neopixelSetColor(uint8_t r, uint8_t g, uint8_t b) {
  uint32_t grb = ((uint32_t)g << 16) | ((uint32_t)r << 8) | b;

  // Disable interrupts
  cli();

  for (int i = 0; i < NUM_LEDS * 24; i++) {
    if (grb & 0x800000) {
      PORTA.OUTSET = PIN5_bm;         // Set pin high
      __builtin_avr_delay_cycles(8);  // ~250 ns delay
      PORTA.OUTCLR = PIN5_bm;         // Set pin low
      __builtin_avr_delay_cycles(4);  // ~125 ns delay
    } else {
      PORTA.OUTSET = PIN5_bm;         // Set pin high
      __builtin_avr_delay_cycles(3);  // ~94 ns delay
      PORTA.OUTCLR = PIN5_bm;         // Set pin low
      __builtin_avr_delay_cycles(9);  // ~281 ns delay
    }
    grb <<= 1;  // Shift to next bit
  }

  // Enable interrupts
  sei();
}

// Function to show the NeoPixel LEDs
void neopixelShow() {
  // Do nothing, as the LEDs are updated immediately in setColor function
}

// Function to clear the NeoPixel LEDs
void neopixelClear() {
  // Clear NeoPixel data
  neopixelSetColor(0, 0, 0);
}
